const React = require('react')
const { render, Box, Color } = require('ink')
const SelectInput = require('ink-select-input').default
const TextInput = require('ink-text-input').default
const Spinner = require('ink-spinner').default

const { getVaultInstance } = require('../vaultUtils')
const generate = require('../generate')

class App extends React.Component {
  constructor () {
    super()

    this.state = {
      processStates: ['token', 'endpoint', 'secretRoom', 'path', 'secret', 'finish'],
      currentProcessState: 'token',
      token: '',
      endpoint: '',
      secretRoom: '',
      path: '',
      isShowSelect: false,
      isLoading: false,
      project: '',
      secretList: []
    }

    this.getVaultSecretsList = this.getVaultSecretsList.bind(this)
    this.getComponentByProcessState = this.getComponentByProcessState.bind(this)
  }

  getComponentByProcessState () {
    const { currentProcessState } = this.state
    const processStateToComponent = {
      token: (
        <Box>
          <Box marginLeft={1}>Enter a token:</Box>
          <TextInput
            value={this.state.token}
            onChange={val => this.handleChange({ key: 'token', value: val })}
            onSubmit={() => this.handleSubmit('endpoint')}
          />
        </Box>
      ),
      endpoint: (
        <Box>
          <Box marginLeft={1}>Enter an endpoint url:</Box>
          <TextInput
            value={this.state.endpoint}
            onChange={val => this.handleChange({ key: 'endpoint', value: val })}
            onSubmit={() => this.handleSubmit('secretRoom')}
          />
        </Box>
      ),
      secretRoom: (
        <Box>
          <Box marginLeft={1}>Enter a secret room:</Box>
          <TextInput
            value={this.state.secretRoom}
            onChange={val => this.handleChange({ key: 'secretRoom', value: val })}
            onSubmit={() => this.handleSubmit('path', this.getVaultSecretsList)}
          />
        </Box>
      ),
      path: (
        <Box>
          <Box marginLeft={1}>Enter a path to where generate (current directory by default):</Box>
          <TextInput
            value={this.state.path}
            onChange={val => this.handleChange({ key: 'path', value: val })}
            onSubmit={() => this.handleSubmit('secret')}
          />
        </Box>
      ),
      secret: (
        <Box>
          <Box marginLeft={1}>Select project:</Box>
          <SelectInput
            items={this.state.secretList}
            onSelect={project => this.handleSelectProject(project)}
          />
        </Box>
      ),
      finish: (
        <Box>
          <Box />
          <Box marginLeft={1}>
            <Color green>successfully written</Color>
          </Box>
        </Box>
      )
    }

    return processStateToComponent[currentProcessState]
  }

  render () {
    if (this.state.isLoading === true) {
      return (
        <Box>
          <Color green>
            <Spinner type="dots" />
          </Color>
        </Box>
      )
    }

    const Component = this.getComponentByProcessState()
    return Component
  }

  handleChange (obj) {
    this.setState({ [obj.key]: obj.value })
  }

  handleSubmit (nextProcessState, callback) {
    this.setState(
      {
        currentProcessState: nextProcessState
      },
      () => {
        if (callback !== undefined) {
          this.setState({ isLoading: true })
          callback()
        }
      }
    )
  }

  getVaultSecretsList () {
    const run = async () => {
      try {
        const vaultInstance = getVaultInstance({
          endpoint: this.state.endpoint,
          token: this.state.token
        })
        var secretList = await vaultInstance.list(this.state.secretRoom)
        var list = secretList.data.keys.map(secret => {
          return { label: secret, value: secret }
        })
        this.setState({ secretList: list })
        this.setState({ isLoading: false })
      } catch (err) {
        console.log(err.message)
        unmount()
      }
    }
    run()
  }

  handleSelectProject (project) {
    this.setState({ isLoading: true })
    const run = async () => {
      generate(
        {
          endpoint: this.state.endpoint,
          token: this.state.token
        },
        project.value,
        this.state.secretRoom,
        this.state.path || '.'
      )
      this.setState({ isLoading: false, currentProcessState: 'finish' })
      unmount()
    }
    run()
  }
}

const { unmount } = render(<App />)
